// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0.If a copy of the MPL was not distributed with this
// file, You can obtain one at https ://mozilla.org/MPL/2.0/.

#ifndef ANGLE_H
#define ANGLE_H

namespace Passer {
namespace LinearAlgebra {

template <typename T>
class AngleOf {
 public:
  AngleOf() {};
  AngleOf(int f);
  AngleOf(float f);
  operator float() const;

  inline T GetBinary() const { return this->value; }

  static AngleOf<T> Rad2Deg;
  static AngleOf<T> Deg2Rad;

  static AngleOf<T> pi;

  bool operator==(AngleOf<T> a);

  static AngleOf<T> Normalize(AngleOf<T> a);
  static AngleOf<T> Clamp(AngleOf<T> a, AngleOf<T> min, AngleOf<T> max);
  static AngleOf<T> Difference(AngleOf<T> a, AngleOf<T> b) {
    AngleOf<T> r = Normalize(b - a);
    return r;
  };
  static AngleOf<T> MoveTowards(AngleOf<T> fromAngle,
                                AngleOf<T> toAngle,
                                AngleOf<T> maxAngle);

  static AngleOf<T> CosineRuleSide(float a, float b, AngleOf<T> gamma);
  static AngleOf<T> CosineRuleAngle(float a, float b, float c);

  static AngleOf<T> SineRuleAngle(float a, AngleOf<T> beta, float c);

 private:
  T value;
};

using Angle = AngleOf<float>;
// using Angle = AngleOf<signed short>;

}  // namespace LinearAlgebra
}  // namespace Passer
using namespace Passer::LinearAlgebra;

#endif